@@ -1,4 +1,10 @@
+"""
+This module provides an interface for invoking CMake executable.
+"""
+
 import argparse
+import distutils.sysconfig as du_sysconfig
+import glob
 import itertools
 import os
 import os.path
@@ -9,33 +15,29 @@ import shlex
 import sys
 import sysconfig
 
-from subprocess import CalledProcessError
-
+from .constants import (CMAKE_BUILD_DIR,
+                        CMAKE_INSTALL_DIR,
+                        SETUPTOOLS_INSTALL_DIR)
 from .platform_specifics import get_platform
 from .exceptions import SKBuildError
 
-SKBUILD_DIR = "_skbuild"
-CMAKE_BUILD_DIR = os.path.join(SKBUILD_DIR, "cmake-build")
-CMAKE_INSTALL_DIR = os.path.join(SKBUILD_DIR, "cmake-install")
-SETUPTOOLS_INSTALL_DIR = os.path.join(SKBUILD_DIR, "setuptools")
-
 RE_FILE_INSTALL = re.compile(
     r"""[ \t]*file\(INSTALL DESTINATION "([^"]+)".*"([^"]+)"\).*""")
 
 
-def pop_arg(arg, a, default=None):
-    """Pops an arg(ument) from an argument list a and returns the new list
-    and the value of the argument if present and a default otherwise.
+def pop_arg(arg, args, default=None):
+    """Pops an argument ``arg`` from an argument list ``args`` and returns the
+    new list and the value of the argument if present and a default otherwise.
     """
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(add_help=False)
     parser.add_argument(arg)
-    ns, a = parser.parse_known_args(a)
-    ns = tuple(vars(ns).items())
-    if len(ns) > 0 and ns[0][1] is not None:
-        val = ns[0][1]
+    namespace, args = parser.parse_known_args(args)
+    namespace = tuple(vars(namespace).items())
+    if len(namespace) > 0 and namespace[0][1] is not None:
+        val = namespace[0][1]
     else:
         val = default
-    return a, val
+    return args, val
 
 
 def _remove_cwd_prefix(path):
@@ -53,52 +55,59 @@ def _remove_cwd_prefix(path):
     return result
 
 
-def _touch_init(folder):
-    init = os.path.join(folder, "__init__.py")
-    if not os.path.exists(init):
-        with open(init, "w") as f:
-            f.write("\n")
-    return _remove_cwd_prefix(init)
-
-
 class CMaker(object):
+    """Interface to CMake executable."""
 
-    def __init__(self, **defines):
+    def __init__(self):
         # verify that CMake is installed
         try:
             subprocess.check_call(['cmake', '--version'])
-        except (OSError, CalledProcessError):
-            raise SKBuildError('CMake is not installed, aborting build.')
+        except (OSError, subprocess.CalledProcessError):
+            raise SKBuildError(
+                "Problem with the CMake installation, aborting build.")
 
         self.platform = get_platform()
 
-    def configure(self, clargs=(), generator_id=None):
+    def configure(self, clargs=(), generator_name=None,
+                  cmake_source_dir='.', cmake_install_dir='', cleanup=True):
         """Calls cmake to generate the Makefile/VS Solution/XCode project.
 
-        Input:
-        ------
-        generator_id: string
+        clargs: tuple
+            List of command line arguments to pass to cmake executable.
+
+        generator_name: string
             The string representing the CMake generator to use.
             If None, uses defaults for your platform.
+
+        cmake_source_dir: string
+            Path to source tree containing a ``CMakeLists.txt``
+
+        cmake_install_dir: string
+            Relative directory to append
+            to :const:`skbuild.constants.CMAKE_INSTALL_DIR`.
+
+        cleanup: bool
+            If True, cleans up temporary folder used to test
+            generators. Set to False for debugging to see CMake's
+            output files.
+
+        Return a mapping of the environment associated with the
+        selected :class:`skbuild.platform_specifics.abstract.CMakeGenerator`.
         """
 
-        # if no provided default generator_id, check environment
-        if generator_id is None:
-            generator_id = os.environ.get("CMAKE_GENERATOR")
+        # if no provided default generator_name, check environment
+        if generator_name is None:
+            generator_name = os.environ.get("CMAKE_GENERATOR")
 
-        # if generator_id is provided on command line, use it
-        clargs, cli_generator_id = pop_arg('-G', clargs)
-        if cli_generator_id is not None:
-            generator_id = cli_generator_id
+        # if generator_name is provided on command line, use it
+        clargs, cli_generator_name = pop_arg('-G', clargs)
+        if cli_generator_name is not None:
+            generator_name = cli_generator_name
 
-        # use the generator_id returned from the platform, with the current
-        # generator_id as a suggestion
-        generator_id = self.platform.get_best_generator(generator_id)
-
-        if generator_id is None:
-            raise SKBuildError(
-                "Could not get working generator for your system."
-                "  Aborting build.")
+        # use the generator returned from the platform, with the current
+        # generator_name as a suggestion
+        generator = self.platform.get_best_generator(
+            generator_name, cmake_args=clargs, cleanup=cleanup)
 
         if not os.path.exists(CMAKE_BUILD_DIR):
             os.makedirs(CMAKE_BUILD_DIR)
@@ -113,11 +122,12 @@ class CMaker(object):
         python_include_dir = CMaker.get_python_include_dir(python_version)
         python_library = CMaker.get_python_library(python_version)
 
-        cwd = os.getcwd()
+        cmake_source_dir = os.path.abspath(cmake_source_dir)
         cmd = [
-            'cmake', cwd, '-G', generator_id,
+            'cmake', cmake_source_dir, '-G', generator.name,
             ("-DCMAKE_INSTALL_PREFIX:PATH=" +
-                os.path.join(cwd, CMAKE_INSTALL_DIR)),
+                os.path.abspath(
+                    os.path.join(CMAKE_INSTALL_DIR, cmake_install_dir))),
             ("-DPYTHON_EXECUTABLE:FILEPATH=" +
                 sys.executable),
             ("-DPYTHON_VERSION_STRING:STRING=" +
@@ -141,7 +151,7 @@ class CMaker(object):
 
         # changes dir to cmake_build and calls cmake's configure step
         # to generate makefile
-        rtn = subprocess.call(cmd, cwd=CMAKE_BUILD_DIR)
+        rtn = subprocess.call(cmd, cwd=CMAKE_BUILD_DIR, env=generator.env)
         if rtn != 0:
             raise SKBuildError(
                 "An error occurred while configuring with CMake.\n"
@@ -153,13 +163,16 @@ class CMaker(object):
                 "    {}\n"
                 "Please see CMake's output for more information.".format(
                     self._formatArgsForDisplay(cmd),
-                    os.path.abspath(cwd),
+                    os.path.abspath(cmake_source_dir),
                     os.path.abspath(CMAKE_BUILD_DIR)))
 
         CMaker.check_for_bad_installs()
 
+        return generator.env
+
     @staticmethod
     def get_python_version():
+        """Get version associated with the current python interpreter."""
         python_version = sysconfig.get_config_var('VERSION')
 
         if not python_version:
@@ -174,6 +187,8 @@ class CMaker(object):
     # need them for this function.
     @staticmethod  # noqa: C901
     def get_python_include_dir(python_version):
+        """Get include directory associated with the current python
+        interpreter."""
         # determine python include dir
         python_include_dir = sysconfig.get_config_var('INCLUDEPY')
 
@@ -185,7 +200,6 @@ class CMaker(object):
         )
 
         if not found_python_h:
-            candidate_prefixes = []
 
             # NOTE(opadron): these possible prefixes must be guarded against
             # AttributeErrors and KeyErrors because they each can throw on
@@ -256,6 +270,8 @@ class CMaker(object):
 
     @staticmethod
     def get_python_library(python_version):
+        """Get path to the python library associated with the current python
+        interpreter."""
         # determine direct path to libpython
         python_library = sysconfig.get_config_var('LIBRARY')
 
@@ -280,7 +296,12 @@ class CMaker(object):
             if abiflags:
                 candidate_abiflags.append('')
 
-            libdir = sysconfig.get_config_var('LIBDIR')
+            # Ensure the value injected by virtualenv is
+            # returned on windows.
+            # Because calling `sysconfig.get_config_var('multiarchsubdir')`
+            # returns an empty string on Linux, `du_sysconfig` is only used to
+            # get the value of `LIBDIR`.
+            libdir = du_sysconfig.get_config_var('LIBDIR')
             if sysconfig.get_config_var('MULTIARCH'):
                 masd = sysconfig.get_config_var('multiarchsubdir')
                 if masd:
@@ -290,7 +311,7 @@ class CMaker(object):
 
             if libdir is None:
                 libdir = os.path.abspath(os.path.join(
-                        sysconfig.get_config_var('LIBDEST'), "..", "libs"))
+                    sysconfig.get_config_var('LIBDEST'), "..", "libs"))
 
             candidates = (
                 os.path.join(
@@ -317,14 +338,21 @@ class CMaker(object):
 
     @staticmethod
     def check_for_bad_installs():
-        # Try to catch files that are meant to be installed outside the project
-        # root before they are actually installed.  We can not wait for the
-        # manifest, so we try to extract the information from the CMake build
-        # files.
+        """This function tries to catch files that are meant to be installed
+        outside the project root before they are actually installed.
+
+        Indeed, we can not wait for the manifest, so we try to extract the
+        information (install destination) from the CMake build files
+        ``*.cmake`` found in ``CMAKE_BUILD_DIR``.
+
+        It raises ``SKBuildError`` if it found install destination outside of
+        ``CMAKE_INSTALL_DIR``.
+        """
+
         bad_installs = []
         install_dir = os.path.join(os.getcwd(), CMAKE_INSTALL_DIR)
 
-        for root, dir_list, file_list in os.walk(CMAKE_BUILD_DIR):
+        for root, _, file_list in os.walk(CMAKE_BUILD_DIR):
             for filename in file_list:
                 if os.path.splitext(filename)[1] != ".cmake":
                     continue
@@ -356,7 +384,7 @@ class CMaker(object):
                     ("      " + _install) for _install in bad_installs)
             )))
 
-    def make(self, clargs=(), config="Release", source_dir="."):
+    def make(self, clargs=(), config="Release", source_dir=".", env=None):
         """Calls the system-specific make program to compile code.
         """
         clargs, config = pop_arg('--config', clargs, config)
@@ -366,14 +394,14 @@ class CMaker(object):
                                 "make?").format(CMAKE_BUILD_DIR))
 
         cmd = ["cmake", "--build", source_dir,
-               "--target", "install", "--config", config]
+               "--target", "install", "--config", config, "--"]
         cmd.extend(clargs)
         cmd.extend(
             filter(bool,
                    shlex.split(os.environ.get("SKBUILD_BUILD_OPTIONS", "")))
         )
 
-        rtn = subprocess.call(cmd, cwd=CMAKE_BUILD_DIR)
+        rtn = subprocess.call(cmd, cwd=CMAKE_BUILD_DIR, env=env)
         if rtn != 0:
             raise SKBuildError(
                 "An error occurred while building with CMake.\n"
@@ -389,14 +417,18 @@ class CMaker(object):
                     os.path.abspath(CMAKE_BUILD_DIR)))
 
     def install(self):
-        """Returns a list of tuples of (install location, file list) to install
-        via setuptools that is compatible with the data_files keyword argument.
+        """Returns a list of file paths to install via setuptools that is
+        compatible with the data_files keyword argument.
         """
-        return self._parse_manifest()
+        return self._parse_manifests()
 
-    def _parse_manifest(self):
-        install_manifest_path = os.path.join(CMAKE_BUILD_DIR,
-                                             "install_manifest.txt")
+    def _parse_manifests(self):
+        paths = \
+            glob.glob(os.path.join(CMAKE_BUILD_DIR, "install_manifest*.txt"))
+        return [self._parse_manifest(path) for path in paths][0]
+
+    @staticmethod
+    def _parse_manifest(install_manifest_path):
         with open(install_manifest_path, "r") as manifest:
             return [_remove_cwd_prefix(path) for path in manifest]
 
