@@ -13,7 +13,7 @@ from contextlib import contextmanager
 from distutils.errors import (DistutilsArgError,
                               DistutilsError,
                               DistutilsGetoptError)
-from shutil import copyfile
+from shutil import copyfile, copymode
 
 # XXX If 'six' becomes a dependency, use 'six.StringIO' instead.
 try:
@@ -25,9 +25,12 @@ from setuptools import setup as upstream_setup
 from setuptools.dist import Distribution as upstream_Distribution
 
 from . import cmaker
-from .command import build, install, clean, bdist, bdist_wheel, egg_info, sdist
+from .command import (build, build_py, clean,
+                      install, install_lib, install_scripts,
+                      bdist, bdist_wheel, egg_info,
+                      sdist, generate_source_manifest)
 from .constants import CMAKE_INSTALL_DIR
-from .exceptions import SKBuildError
+from .exceptions import SKBuildError, SKBuildGeneratorNotFoundError
 from .utils import (mkdir_p, PythonModuleFinder, to_platform_path, to_unix_path)
 
 
@@ -113,12 +116,20 @@ def _parse_setuptools_arguments(setup_attrs):
     """This function instantiates a Distribution object and
     parses the command line arguments.
 
-    It returns a tuple (display_only, help_commands, commands) where
-     - display_only is a boolean indicating if an argument like '--help',
-     '--help-commands' or '--author' was passed.
-     - help_commands is a boolean indicating if argument '--help-commands'
-     was passed.
-     - commands contains the list of commands that were passed.
+    It returns the tuple
+        ``(display_only, help_commands, commands,
+        hide_listing, force_cmake, skip_cmake)``
+    where
+    - display_only is a boolean indicating if an argument like '--help',
+      '--help-commands' or '--author' was passed.
+    - help_commands is a boolean indicating if argument '--help-commands'
+      was passed.
+    - commands contains the list of commands that were passed.
+    - hide_listing is a boolean indicating if the list of files being included
+      in the distribution is displayed or not.
+    - force_cmake a boolean indicating that CMake should always be executed.
+    - skip_cmake is a boolean indicating if the execution of CMake should
+      explicitly be skipped.
 
     Otherwise it raises DistutilsArgError exception if there are
     any error on the command-line, and it raises DistutilsGetoptError
@@ -133,6 +144,22 @@ def _parse_setuptools_arguments(setup_attrs):
 
     dist = upstream_Distribution(setup_attrs)
 
+    # Update class attribute to also ensure the argument is processed
+    # when ``upstream_setup`` is called.
+    # pylint:disable=no-member
+    upstream_Distribution.global_options.append(
+        ('hide-listing', None, "do not display list of files being "
+                               "included in the distribution")
+    )
+    # pylint:disable=no-member
+    upstream_Distribution.global_options.append(
+        ('force-cmake', None, "always run CMake")
+    )
+    # pylint:disable=no-member
+    upstream_Distribution.global_options.append(
+        ('skip-cmake', None, "do not run CMake")
+    )
+
     # Find and parse the config file(s): they will override options from
     # the setup script, but be overridden by the command line.
     dist.parse_config_files()
@@ -144,8 +171,15 @@ def _parse_setuptools_arguments(setup_attrs):
     with _capture_output():
         result = dist.parse_command_line()
         display_only = not result
+        if not hasattr(dist, 'hide_listing'):
+            dist.hide_listing = False
+        if not hasattr(dist, 'force_cmake'):
+            dist.force_cmake = False
+        if not hasattr(dist, 'skip_cmake'):
+            dist.skip_cmake = False
 
-    return display_only, dist.help_commands, dist.commands
+    return (display_only, dist.help_commands, dist.commands,
+            dist.hide_listing, dist.force_cmake, dist.skip_cmake)
 
 
 def _check_skbuild_parameters(skbuild_kw):
@@ -188,8 +222,9 @@ def strip_package(package_parts, module_file):
 
     module_dir = module_dir[:len(package)]
 
-    return module_file[len(package) + 1:] if module_dir.startswith(
-        package) else module_file
+    return (module_file[len(package) + 1:]
+            if package != "" and module_dir.startswith(package)
+            else module_file)
 
 
 def _package_data_contain_module(module, package_data):
@@ -201,7 +236,7 @@ def _package_data_contain_module(module, package_data):
     """
     (package, _, module_file) = module
     if package not in package_data:
-        return True
+        return False
     # We need to strip the package because a module entry
     # usually looks like this:
     #
@@ -211,12 +246,34 @@ def _package_data_contain_module(module, package_data):
     #
     #   {'foo.bar' : ['module.py']}
     if (strip_package(package.split("."), module_file)
-            not in package_data[package]):
+            in package_data[package]):
         return True
     return False
 
 
-# pylint:disable=too-many-locals
+def _should_run_cmake(commands, cmake_with_sdist):
+    """Return True if at least one command requiring ``cmake`` to run
+    is found in ``commands``."""
+    for expected_command in [
+        "build",
+        "develop",
+        "install",
+        "install_lib",
+        "bdist",
+        "bdist_dumb",
+        "bdist_egg"
+        "bdist_rpm",
+        "bdist_wininst",
+        "bdist_wheel",
+    ]:
+        if expected_command in commands:
+            return True
+    if "sdist" in commands and cmake_with_sdist:
+        return True
+    return False
+
+
+# pylint:disable=too-many-locals, too-many-branches
 def setup(*args, **kw):  # noqa: C901
     """This function wraps setup() so that we can run cmake, make,
     CMake build, then proceed as usual with setuptools, appending the
@@ -228,13 +285,21 @@ def setup(*args, **kw):  # noqa: C901
     # (patches provided, but no updates since 2014)
     cmdclass = kw.get('cmdclass', {})
     cmdclass['build'] = cmdclass.get('build', build.build)
+    cmdclass['build_py'] = cmdclass.get('build_py', build_py.build_py)
     cmdclass['install'] = cmdclass.get('install', install.install)
+    cmdclass['install_lib'] = cmdclass.get('install_lib',
+                                           install_lib.install_lib)
+    cmdclass['install_scripts'] = cmdclass.get('install_scripts',
+                                               install_scripts.install_scripts)
     cmdclass['clean'] = cmdclass.get('clean', clean.clean)
     cmdclass['sdist'] = cmdclass.get('sdist', sdist.sdist)
     cmdclass['bdist'] = cmdclass.get('bdist', bdist.bdist)
     cmdclass['bdist_wheel'] = cmdclass.get(
         'bdist_wheel', bdist_wheel.bdist_wheel)
     cmdclass['egg_info'] = cmdclass.get('egg_info', egg_info.egg_info)
+    cmdclass['generate_source_manifest'] = cmdclass.get(
+        'generate_source_manifest',
+        generate_source_manifest.generate_source_manifest)
     kw['cmdclass'] = cmdclass
 
     # Extract setup keywords specific to scikit-build and remove them from kw.
@@ -244,7 +309,8 @@ def setup(*args, **kw):  # noqa: C901
     parameters = {
         'cmake_args': [],
         'cmake_install_dir': '',
-        'cmake_source_dir': ''
+        'cmake_source_dir': '',
+        'cmake_with_sdist': False
     }
     skbuild_kw = {param: kw.pop(param, parameters[param])
                   for param in parameters}
@@ -268,13 +334,17 @@ def setup(*args, **kw):  # noqa: C901
         cmake_source_dir = os.path.relpath(cmake_source_dir)
 
     # Skip running CMake in the following cases:
+    # * flag "--skip-cmake" is provided
+    # * "display only" argument is provided (e.g  '--help', '--author', ...)
     # * no command-line arguments or invalid ones are provided
-    # * "display only" argument like '--help', '--help-commands'
-    #   or '--author' are provided
+    # * no command requiring cmake is provided
+    # * no CMakeLists.txt if found
     display_only = has_invalid_arguments = help_commands = False
+    force_cmake = skip_cmake = False
     commands = []
     try:
-        (display_only, help_commands, commands) = \
+        (display_only, help_commands, commands,
+         hide_listing, force_cmake, skip_cmake) = \
             _parse_setuptools_arguments(kw)
     except (DistutilsArgError, DistutilsGetoptError):
         has_invalid_arguments = True
@@ -284,13 +354,13 @@ def setup(*args, **kw):  # noqa: C901
     if not has_cmakelists:
         print('skipping skbuild (no CMakeLists.txt found)')
 
-    skip_cmake = (display_only
+    skip_cmake = (skip_cmake
+                  or display_only
                   or has_invalid_arguments
-                  or 'clean' in commands
-                  or 'egg_info' in commands
-                  or 'sdist' in commands
+                  or not _should_run_cmake(commands,
+                                           skbuild_kw["cmake_with_sdist"])
                   or not has_cmakelists)
-    if skip_cmake:
+    if skip_cmake and not force_cmake:
         if help_commands:
             # Prepend scikit-build help. Generate option descriptions using
             # argparse.
@@ -309,6 +379,8 @@ def setup(*args, **kw):  # noqa: C901
             print('')
         return upstream_setup(*args, **kw)
 
+    developer_mode = "develop" in commands
+
     packages = kw.get('packages', [])
     package_dir = kw.get('package_dir', {})
     package_data = kw.get('package_data', {}).copy()
@@ -331,10 +403,12 @@ def setup(*args, **kw):  # noqa: C901
 
     try:
         cmkr = cmaker.CMaker()
-        cmkr.configure(cmake_args,
-                       cmake_source_dir=cmake_source_dir,
-                       cmake_install_dir=skbuild_kw['cmake_install_dir'])
-        cmkr.make(make_args)
+        env = cmkr.configure(cmake_args,
+                             cmake_source_dir=cmake_source_dir,
+                             cmake_install_dir=skbuild_kw['cmake_install_dir'])
+        cmkr.make(make_args, env=env)
+    except SKBuildGeneratorNotFoundError as ex:
+        sys.exit(ex)
     except SKBuildError as ex:
         import traceback
         print("Traceback (most recent call last):")
@@ -355,8 +429,24 @@ def setup(*args, **kw):  # noqa: C901
                     data_files,
                     cmake_source_dir, skbuild_kw['cmake_install_dir'])
 
-    _consolidate(cmake_source_dir,
-                 packages, package_dir, py_modules, package_data)
+    if developer_mode:
+        # Copy packages
+        for package, package_file_list in package_data.items():
+            for package_file in package_file_list:
+                package_file = os.path.join(package_dir[package], package_file)
+                cmake_file = os.path.join(CMAKE_INSTALL_DIR, package_file)
+                if os.path.exists(cmake_file):
+                    _copy_file(cmake_file, package_file, hide_listing)
+        # Copy modules
+        for py_module in py_modules:
+            package_file = py_module + ".py"
+            cmake_file = os.path.join(CMAKE_INSTALL_DIR, package_file)
+            if os.path.exists(cmake_file):
+                _copy_file(cmake_file, package_file, hide_listing)
+    else:
+        _consolidate(cmake_source_dir,
+                     packages, package_dir, py_modules, package_data,
+                     hide_listing)
 
     kw['package_data'] = package_data
     kw['package_dir'] = {
@@ -367,11 +457,6 @@ def setup(*args, **kw):  # noqa: C901
         for prefix, package in package_prefixes
     }
 
-    kw['py_modules'] = [
-        os.path.join(CMAKE_INSTALL_DIR, py_module) if mask else py_module
-        for py_module, mask in new_py_modules.items()
-    ]
-
     kw['scripts'] = [
         os.path.join(CMAKE_INSTALL_DIR, script) if mask else script
         for script, mask in new_scripts.items()
@@ -386,7 +471,7 @@ def setup(*args, **kw):  # noqa: C901
     # pylint: disable=missing-docstring
     class BinaryDistribution(upstream_Distribution):
         def has_ext_modules(self):  # pylint: disable=no-self-use
-            return True
+            return has_cmakelists
     kw['distclass'] = BinaryDistribution
 
     print("")
@@ -480,7 +565,7 @@ def _classify_files(install_paths, package_data, package_prefixes,
 
         # check to see if path is part of a package
         for prefix, package in package_prefixes:
-            if path.startswith(prefix):
+            if path.startswith(prefix + "/"):
                 # peel off the package prefix
                 path = os.path.relpath(path, prefix)
 
@@ -532,8 +617,32 @@ def _classify_files(install_paths, package_data, package_prefixes,
         del parent_dir, file_set
 
 
+def _copy_file(src_file, dest_file, hide_listing=True):
+    """Copy ``src_file`` to ``dest_file`` ensuring parent directory exists.
+
+    By default, message like `creating directory /path/to/package` and
+    `copying directory /src/path/to/package -> path/to/package` are displayed
+    on standard output. Setting ``hide_listing`` to False avoids message from
+    being displayed.
+    """
+    # Create directory if needed
+    dest_dir = os.path.dirname(dest_file)
+    if dest_dir != "" and not os.path.exists(dest_dir):
+        if not hide_listing:
+            print("creating directory {}".format(dest_dir))
+        mkdir_p(dest_dir)
+
+    # Copy file
+    if not hide_listing:
+        print("copying {} -> {}".format(src_file, dest_file))
+    copyfile(src_file, dest_file)
+    copymode(src_file, dest_file)
+
+
 def _consolidate(
-        cmake_source_dir, packages, package_dir, py_modules, package_data):
+        cmake_source_dir, packages, package_dir, py_modules, package_data,
+        hide_listing
+):
     """This function consolidates packages having modules located in
     both the source tree and the CMake install tree into one location.
 
@@ -573,23 +682,15 @@ def _consolidate(
     for entry in modules:
 
         # Check if module file should be copied into the CMake install tree.
-        if not _package_data_contain_module(entry, package_data):
+        if _package_data_contain_module(entry, package_data):
             continue
 
         (package, _, src_module_file) = entry
 
         # Copy missing module file
-        dest_module_file = os.path.join(CMAKE_INSTALL_DIR, src_module_file)
-
-        # Create directory if needed
-        dest_module_dir = os.path.dirname(dest_module_file)
-        if not os.path.exists(dest_module_dir):
-            print("creating directory {}".format(dest_module_dir))
-            mkdir_p(dest_module_dir)
-
-        # Copy file
-        print("copying {} -> {}".format(src_module_file, dest_module_file))
-        copyfile(src_module_file, dest_module_file)
+        if os.path.exists(src_module_file):
+            dest_module_file = os.path.join(CMAKE_INSTALL_DIR, src_module_file)
+            _copy_file(src_module_file, dest_module_file, hide_listing)
 
         # Since the mapping in package_data expects the package to be associated
         # with a list of files relative to the directory containing the package,
