@@ -117,44 +117,55 @@ def _copy_dir(target_dir, entry, on_duplicate='exception', keep_top_dir=False):
             # on_duplicate == 'ignore': do nothing with e2
 
 
-def initialize_git_repo_and_commit(project_dir):
+def initialize_git_repo_and_commit(project_dir, verbose=True):
     """Convenience function creating a git repository in ``project_dir``.
 
     If ``project_dir`` does NOT contain a ``.git`` directory, a new
     git repository with one commit containing all the directories and files
     is created.
     """
+    if isinstance(project_dir, six.string_types):
+        project_dir = py.path.local(project_dir)
+
     if project_dir.join('.git').exists():
         return
 
+    # If any, exclude virtualenv files
+    project_dir.join(".gitignore").write(".env")
+
     with push_dir(str(project_dir)):
         for cmd in [
             ['git', 'init'],
             ['git', 'config', 'user.name', 'scikit-build'],
             ['git', 'config', 'user.email', 'test@test'],
             ['git', 'add', '-A'],
+            ['git', 'reset', '.gitignore'],
             ['git', 'commit', '-m', 'Initial commit']
         ]:
-            subprocess.check_call(cmd)
+            do_call = (subprocess.check_call
+                       if verbose else subprocess.check_output)
+            do_call(cmd)
 
 
-def prepare_project(project, tmp_project_dir):
+def prepare_project(project, tmp_project_dir, force=False):
     """Convenience function setting up the build directory ``tmp_project_dir``
     for the selected sample ``project``.
 
     If ``tmp_project_dir`` does not exist, it is created.
 
     If ``tmp_project_dir`` is empty, the sample ``project`` is copied into it.
+    Specifying ``force=True`` will copy the files even if ``tmp_project_dir``
+    is not empty.
     """
-
-    tmp_project_dir = py.path.local(tmp_project_dir)
+    if isinstance(tmp_project_dir, six.string_types):
+        tmp_project_dir = py.path.local(tmp_project_dir)
 
     # Create project directory if it does not exist
     if not tmp_project_dir.exists():
         tmp_project_dir = _tmpdir(project)
 
-    # If empty, copy project files and initialize git
-    if not tmp_project_dir.listdir():
+    # If empty or if force is True, copy project files and initialize git
+    if not tmp_project_dir.listdir() or force:
         _copy_dir(tmp_project_dir, os.path.join(SAMPLES_DIR, project))
 
 
@@ -179,7 +190,7 @@ def execute_setup_py(project_dir, setup_args):
         yield
 
 
-def project_setup_py_test(project, setup_args, tmp_dir=None):
+def project_setup_py_test(project, setup_args, tmp_dir=None, verbose_git=True):
 
     def dec(fun):
 
@@ -189,7 +200,8 @@ def project_setup_py_test(project, setup_args, tmp_dir=None):
             if wrapped.tmp_dir is None:
                 wrapped.tmp_dir = _tmpdir(fun.__name__)
                 prepare_project(wrapped.project, wrapped.tmp_dir)
-                initialize_git_repo_and_commit(wrapped.tmp_dir)
+                initialize_git_repo_and_commit(
+                    wrapped.tmp_dir, verbose=wrapped.verbose_git)
 
             with execute_setup_py(wrapped.tmp_dir, wrapped.setup_args):
                 result2 = fun(*iargs, **ikwargs)
@@ -199,7 +211,54 @@ def project_setup_py_test(project, setup_args, tmp_dir=None):
         wrapped.project = project
         wrapped.setup_args = setup_args
         wrapped.tmp_dir = tmp_dir
+        wrapped.verbose_git = verbose_git
 
         return wrapped
 
     return dec
+
+
+def get_cmakecache_variables(cmakecache):
+    """Returns a dictionary of all variables found in given CMakeCache.txt.
+
+    Dictionary entries are tuple of the
+    form ``(variable_type, variable_value)``.
+
+    Possible `variable_type` are documented
+    `here <https://cmake.org/cmake/help/v3.7/prop_cache/TYPE.html>`_.
+    """
+    results = {}
+    cache_entry_pattern = re.compile(r"^([\w\d_-]+):([\w]+)=")
+    with open(cmakecache) as content:
+        for line in content.readlines():
+            line = line.strip()
+            result = cache_entry_pattern.match(line)
+            if result:
+                variable_name = result.group(1)
+                variable_type = result.group(2)
+                variable_value = line.split("=")[1]
+                results[variable_name] = (variable_type, variable_value)
+    return results
+
+
+def which(name, flags=os.X_OK):
+    """Analogue of unix 'which'. Borrowed from the Twisted project, see
+       their licence here: https://twistedmatrix.com/trac/browser/trunk/LICENSE
+
+       Copied from ``pytest_shutil.cmdline.which`` to allow testing on
+       conda-forge where ``pytest-shutil`` is not available.
+    """
+    result = []
+    exts = filter(None, os.environ.get('PATHEXT', '').split(os.pathsep))
+    path = os.environ.get('PATH', None)
+    if path is None:
+        return []
+    for p in os.environ.get('PATH', '').split(os.pathsep):
+        p = os.path.join(p, name)
+        if os.access(p, flags):
+            result.append(p)
+        for e in exts:
+            pext = p + e
+            if os.access(pext, flags):
+                result.append(pext)
+    return result
