@@ -21,7 +21,8 @@ from skbuild.exceptions import SKBuildError
 from skbuild.setuptools_wrap import strip_package
 from skbuild.utils import (push_dir, to_platform_path)
 
-from . import (_tmpdir, execute_setup_py, push_argv)
+from . import (_tmpdir, execute_setup_py,
+               initialize_git_repo_and_commit, push_argv)
 
 
 @pytest.mark.parametrize("distribution_type",
@@ -219,6 +220,45 @@ def test_cmake_install_dir_keyword(
             *"_skbuild/cmake-install/banana/__init__.py".split("/"))) in out
 
 
+@pytest.mark.parametrize("cmake_with_sdist", [True, False])
+def test_cmake_with_sdist_keyword(cmake_with_sdist, capfd):
+    tmp_dir = _tmpdir('cmake_with_sdist')
+
+    tmp_dir.join('setup.py').write(textwrap.dedent(
+        """
+        from skbuild import setup
+        setup(
+            name="hello",
+            version="1.2.3",
+            description="a minimal example package",
+            author='The scikit-build team',
+            license="MIT",
+            cmake_with_sdist={cmake_with_sdist}
+        )
+        """.format(cmake_with_sdist=cmake_with_sdist)
+    ))
+    tmp_dir.join('CMakeLists.txt').write(textwrap.dedent(
+        """
+        cmake_minimum_required(VERSION 3.5.0)
+        project(test NONE)
+        install(CODE "execute_process(
+          COMMAND \${CMAKE_COMMAND} -E sleep 0)")
+        """
+    ))
+
+    initialize_git_repo_and_commit(tmp_dir)
+
+    with execute_setup_py(tmp_dir, ['sdist']):
+        pass
+
+    out, _ = capfd.readouterr()
+
+    if cmake_with_sdist:
+        assert "Generating done" in out
+    else:
+        assert "Generating done" not in out
+
+
 @pytest.mark.parametrize("distribution_type", ('pure', 'skbuild'))
 def test_script_keyword(distribution_type, capsys):
 
@@ -311,7 +351,7 @@ def test_py_modules_keyword(distribution_type, capsys):
     #     foo.py
     #     bar.py
     #
-    # "SOURCE" tree layout for "pure" distribution:
+    # "SOURCE" tree layout for "skbuild" distribution:
     #
     # ROOT/
     #     setup.py
@@ -381,6 +421,7 @@ def test_py_modules_keyword(distribution_type, capsys):
 
 @pytest.mark.parametrize("package_parts, module_file, expected", [
     ([], "", ""),
+    ([""], "file.py", "file.py"),
     ([], "foo/file.py", "foo/file.py"),
     (["foo"], "", ""),
     (["foo"], "", ""),
@@ -395,12 +436,16 @@ def test_strip_package(package_parts, module_file, expected):
     assert strip_package(package_parts, module_file) == expected
 
 
-@pytest.mark.parametrize("has_cmake_package", [0, 1])
+@pytest.mark.parametrize("has_cmake_package", [0, 1])  # noqa: C901
+@pytest.mark.parametrize("has_cmake_module", [0, 1])
 @pytest.mark.parametrize("has_hybrid_package", [0, 1])
 @pytest.mark.parametrize("has_pure_package", [0, 1])
+@pytest.mark.parametrize("has_pure_module", [0, 1])
 @pytest.mark.parametrize("with_package_base", [0, 1])
 def test_setup_inputs(
-        has_cmake_package, has_hybrid_package, has_pure_package,
+        has_cmake_package, has_cmake_module,
+        has_hybrid_package,
+        has_pure_package, has_pure_module,
         with_package_base,
         mocker):
     """This test that a project can have a package with some modules
@@ -410,9 +455,10 @@ def test_setup_inputs(
     tmp_dir = _tmpdir('test_setup_inputs')
 
     package_base = 'base' if with_package_base else ''
+    package_base_dir = package_base + '/' if package_base else ''
     cmake_source_dir = package_base
 
-    if cmake_source_dir and has_cmake_package:
+    if cmake_source_dir and (has_cmake_package or has_cmake_module):
         pytest.skip("unsupported configuration: "
                     "python package fully generated by CMake does *NOT* work. "
                     "At least __init__.py should be in the project source tree")
@@ -426,6 +472,8 @@ def test_setup_inputs(
     #
     #     [<base>/]
     #
+    #         pureModule.py
+    #
     #         pure/
     #             __init__.py
     #             pure.py
@@ -462,6 +510,8 @@ def test_setup_inputs(
     #
     # ROOT/
     #
+    #     cmakeModule.py                 # CMake
+    #
     #     cmake/
     #         __init__.py                # CMake
     #         cmake.py                   # CMake
@@ -484,6 +534,8 @@ def test_setup_inputs(
     #             hybrid_2_pure_1.py     # CMake or Setuptools
     #             hybrid_2_pure_2.py     # CMake or Setuptools
     #
+    #     pureModule.py                  # Setuptools
+    #
     #     pure/
     #         __init__.py                # Setuptools
     #         pure.py                    # Setuptools
@@ -510,6 +562,10 @@ def test_setup_inputs(
         {h_off}    'hybrid',
         {c_off}    'cmake',
         {p_off}    'hybrid.hybrid_2_pure',
+            ],
+            py_modules=[
+        {pm_off}   '{package_base}pureModule',
+        {cm_off}   '{package_base}cmakeModule',
             ],
             #package_data=[
             #    '': ['*.dat']
@@ -525,10 +581,12 @@ def test_setup_inputs(
         )
         """.format(
             cmake_source_dir=cmake_source_dir,
-            package_base=package_base + '/' if package_base else '',
+            package_base=package_base_dir,
             c_off='' if has_cmake_package else '#',
+            cm_off='' if has_cmake_module else '#',
             h_off='' if has_hybrid_package else '#',
-            p_off='' if has_pure_package else '#'
+            p_off='' if has_pure_package else '#',
+            pm_off='' if has_pure_module else '#'
         )
     ))
 
@@ -549,6 +607,11 @@ def test_setup_inputs(
         {c_off}     DESTINATION cmake
         {c_off}     )
 
+        {cm_off} file(WRITE ${{build_dir}}/cmakeModule.py "")
+        {cm_off} install(
+        {cm_off}     FILES ${{build_dir}}/cmakeModule.py
+        {cm_off}     DESTINATION .)
+
         {h_off} file(WRITE ${{build_dir}}/hybrid_cmake.py "")
         {h_off} install(
         {h_off}     FILES ${{build_dir}}/hybrid_cmake.py
@@ -562,18 +625,21 @@ def test_setup_inputs(
         install(CODE "message(STATUS \\\"Installation complete\\\")")
         """.format(
             c_off='' if has_cmake_package else '#',
+            cm_off='' if has_cmake_module else '#',
             h_off='' if has_hybrid_package else '#',
             p_off='' if has_pure_package else '#'
         )
     ))
 
-    # List path types: 'c', 'h' or 'p'
+    # List path types: 'c', 'cm', 'h', 'p' or 'pm'
     try:
         path_types = list(zip(*filter(
             lambda i: i[1],
             [('c', has_cmake_package),
+             ('cm', has_cmake_module),
              ('h', has_hybrid_package),
-             ('p', has_pure_package)])))[0]
+             ('p', has_pure_package),
+             ('pm', has_pure_module)])))[0]
     except IndexError:
         path_types = []
 
@@ -581,7 +647,7 @@ def test_setup_inputs(
         """Return a filtered list paths considering ``path_types``.
 
         `annotated_paths`` is list of tuple ``(type, path)`` where type
-        is either `c`, `h` or `p`.
+        is either `c`, 'cm', `h`, `p` or 'pm'.
 
         """
         return filter(lambda i: i[0] in path_types, annotated_paths)
@@ -593,6 +659,8 @@ def test_setup_inputs(
         # ('c', 'cmake/__init__.py'),
         # ('c', 'cmake/cmake.py'),
 
+        # ('cm', 'cmakeModule.py'),
+
         ('h', 'hybrid/__init__.py'),
         # ('h', 'hybrid/hybrid_cmake.dat'),
         # ('h', 'hybrid/hybrid_cmake.py'),
@@ -610,17 +678,21 @@ def test_setup_inputs(
         ('p', 'hybrid/hybrid_2_pure/hybrid_2_pure_1.py'),
         ('p', 'hybrid/hybrid_2_pure/hybrid_2_pure_2.py'),
 
+        ('pm', 'pureModule.py'),
+
         ('p', 'pure/__init__.py'),
         ('p', 'pure/pure.py'),
 
         ('p', 'pure/data/pure.dat'),
     ]):
-        assert type in ['p', 'h']
-        root = package_base if type == 'p' else cmake_source_dir
+        assert type in ['p', 'pm', 'h']
+        root = (package_base
+                if (type == 'p' or type == 'pm')
+                else cmake_source_dir)
         tmp_dir.ensure(os.path.join(root, path))
 
     # Do not call the real setup function. Instead, replace it with
-    # a MagicMock allowing to
+    # a MagicMock allowing to check with which arguments it was invoked.
     mock_setup = mocker.patch('skbuild.setuptools_wrap.upstream_setup')
 
     # Convenience print function
@@ -659,22 +731,17 @@ def test_setup_inputs(
         _pprint('expected_package_dir', expected_package_dir)
         _pprint('package_dir')
 
-        def prepend_package_base(module_files):
-            # return [package_base + module_file
-            #         for module_file in module_files]
-            return module_files
-
         # package data
         expected_package_data = {}
 
         if has_cmake_package:
-            expected_package_data['cmake'] = prepend_package_base([
+            expected_package_data['cmake'] = [
                 '__init__.py',
                 'cmake.py'
-            ])
+            ]
 
         if has_hybrid_package:
-            expected_package_data['hybrid'] = prepend_package_base([
+            expected_package_data['hybrid'] = [
                 '__init__.py',
                 # 'hybrid_cmake.dat',
                 'hybrid_cmake.py',
@@ -682,25 +749,33 @@ def test_setup_inputs(
                 'hybrid_pure.py',
                 # 'data/hybrid_data_cmake.dat',
                 # 'data/hybrid_data_pure.dat',
-            ])
-            expected_package_data['hybrid.hybrid_2'] = prepend_package_base([
+            ]
+            expected_package_data['hybrid.hybrid_2'] = [
                 '__init__.py',
                 'hybrid_2_cmake.py',
                 'hybrid_2_pure.py'
-            ])
+            ]
 
         if has_pure_package:
             expected_package_data['hybrid.hybrid_2_pure'] = \
-                prepend_package_base([
+                [
                     '__init__.py',
                     'hybrid_2_pure_1.py',
                     'hybrid_2_pure_2.py'
-                ])
-            expected_package_data['pure'] = prepend_package_base([
+                ]
+            expected_package_data['pure'] = [
                 '__init__.py',
                 'pure.py',
                 # 'data/pure.dat',
-            ])
+            ]
+
+        if has_cmake_module or has_pure_module:
+            expected_modules = []
+            if has_cmake_module:
+                expected_modules.append(package_base_dir + 'cmakeModule.py')
+            if has_pure_module:
+                expected_modules.append(package_base_dir + 'pureModule.py')
+            expected_package_data[''] = expected_modules
 
         _pprint('expected_package_data', expected_package_data)
         package_data = {p: sorted(files)
@@ -708,8 +783,12 @@ def test_setup_inputs(
 
         _pprint('package_data', package_data)
 
-        # py_modules
+        # py_modules (corresponds to files associated with empty package)
         expected_py_modules = []
+        if '' in expected_package_data:
+            expected_py_modules = [
+                os.path.splitext(module_file)[0]
+                for module_file in expected_package_data['']]
         _pprint('expected_py_modules', expected_py_modules)
         _pprint('py_modules')
 
